"use client";
import useShowToast from "@/app/hooks/useShowToast";
import { createUserRequest, editUserRequest } from "@/app/lib/users/action";
import { CheckboxWrapper, InputWrapper } from "@/components/fields/Fields";
import MultiSelectWrapper from "@/components/fields/MultiSelectWrapper";
import FormComponent from "@/components/form/Form";
import prepareValidationSchema from "@/utils/validation-schema";
import {
  Box,
  Button,
  FormControl,
  FormLabel,
  Text,
  useColorModeValue,
} from "@chakra-ui/react";
import { useRouter } from "next/navigation";
import { useRef } from "react";
import { UserInterface } from "../type/user";
import { formFields } from "../variables/formFields";
import { Role } from "../../roles/components/table";
import { ProjectInterface } from "@/types/project";
import { IbcInterface } from "@/types/settings/ibc/ibc";

interface formProps {
  isEdit: UserInterface;
  roles: Role[];
  ibcs: IbcInterface[];
  projects: ProjectInterface[]
}

const UserForm = ({ isEdit, roles, ibcs, projects }: formProps) => {
  const formRef = useRef(null);
  const router = useRouter();
  const showToast = useShowToast();
  const textColor = useColorModeValue("navy.700", "white");
  const brandStars = useColorModeValue("brand.500", "brand.400");

  const formDetails = prepareValidationSchema(formFields);
  const { initialFormData, validationSchema } = formDetails;

  // Set initial values based on whether we are adding or editing
  const initialValues = isEdit ? { ...isEdit } : initialFormData;

  const handleSubmitClick = () => {
    formRef?.current?.externalHandleSubmit();
  };

  const handleCreate = async (values: FormData) => {
    try {
      debugger;
      const response = await createUserRequest(values);
      showToast("User Created!", response.message, "success");
      router.push("/admin/users");
    } catch (error: any) {
      showToast("Failed to create", error.message, "error");
    }
  };

  const handleUpdate = async (values: FormData) => {
    try {
      const payload = { userId: isEdit.id, data: values };
      const response = await await editUserRequest(payload);
      showToast("User Updated!", response.message, "success");
      router.push("/admin/users");
    } catch (error: any) {
      showToast("Failed to create", error.message, "error");
    }
  };

  const handleSubmit = (values: FormData) => {
    if (isEdit) {
      handleUpdate(values);
    } else {
      handleCreate(values);
    }
  };

  return (
    <>
      <FormComponent
        ref={formRef}
        validationSchema={validationSchema}
        initialValues={initialValues}
        // enableReinitialize // Ensures the form reinitializes with updated initial values
        submitForm={handleSubmit}
      >
        <Box display="flex" gap={15}>
          <FormControl w="50%">
            <FormLabel
              fontSize="sm"
              fontWeight="500"
              color={textColor}
              mb="8px"
            >
              First Name &nbsp;
              <Text display="inline-block" color={brandStars}>
                *
              </Text>
            </FormLabel>
            <InputWrapper
              form
              fieldName="first_name"
              defaultValue={isEdit?.first_name}
              id="first_name"
              variant="auth"
              w="full"
              mb="2px"
              fontSize="sm"
              type="text"
              fontWeight="500"
            />
          </FormControl>
          <FormControl w="50%">
            <FormLabel
              fontSize="sm"
              fontWeight="500"
              color={textColor}
              mb="8px"
            >
              Last Name &nbsp;
              <Text display="inline-block" color={brandStars}>
                *
              </Text>
            </FormLabel>
            <InputWrapper
              form
              fieldName="last_name"
              defaultValue={isEdit?.last_name}
              id="last_name"
              variant="auth"
              w="full"
              mb="2px"
              fontSize="sm"
              type="text"
              fontWeight="500"
            />
          </FormControl>
        </Box>
        <Box display="flex" gap={15}>
          <FormControl w="50%">
            <FormLabel
              fontSize="sm"
              fontWeight="500"
              color={textColor}
              mb="8px"
            >
              Email &nbsp;
              <Text display="inline-block" color={brandStars}>
                *
              </Text>
            </FormLabel>
            <InputWrapper
              form
              fieldName="email"
              defaultValue={isEdit?.email}
              id="email"
              variant="auth"
              w="full"
              mb="2px"
              fontSize="sm"
              type="text"
              fontWeight="500"
            />
          </FormControl>
          <FormControl w="50%">
            <FormLabel
              fontSize="sm"
              fontWeight="500"
              color={textColor}
              mb="8px"
            >
              Phone &nbsp;
              <Text display="inline-block" color={brandStars}>
                *
              </Text>
            </FormLabel>
            <InputWrapper
              form
              fieldName="phone"
              defaultValue={isEdit?.phone}
              id="phone"
              variant="auth"
              w="full"
              mb="2px"
              fontSize="sm"
              type="text"
              fontWeight="500"
            />
          </FormControl>
        </Box>
        <Box display="flex" gap={15}>
          <FormControl w="50%">
            <FormLabel
              fontSize="sm"
              fontWeight="500"
              color={textColor}
              mb="8px"
            >
              Password &nbsp;
              <Text display="inline-block" color={brandStars}>
                *
              </Text>
            </FormLabel>
            <InputWrapper
              form
              fieldName="password"
              defaultValue={isEdit?.password ?? ""}
              id="password"
              variant="auth"
              w="full"
              mb="2px"
              fontSize="sm"
              type="text"
              fontWeight="500"
            />
          </FormControl>
          <FormControl w="50%">
            <FormLabel
              fontSize="sm"
              fontWeight="500"
              color={textColor}
              mb="8px"
            >
              Employee Id &nbsp;
              <Text display="inline-block" color={brandStars}>
                *
              </Text>
            </FormLabel>
            <InputWrapper
              form
              fieldName="employ_id"
              defaultValue={isEdit?.employ_id}
              id="employ_id"
              variant="auth"
              w="full"
              mb="2px"
              fontSize="sm"
              type="text"
              fontWeight="500"
            />
          </FormControl>
        </Box>

        <Box display="flex" gap={15}>
          <FormControl w="50%">
            <MultiSelectWrapper
              id="roles"
              fieldName="roles"
              label="Roles"
              options={roles?.map((item: any) => {
                return { value: item.id, label: item.name };
              })}
              defaultValue={
                isEdit?.roles?.map((item: any) => ({
                  value: item.id,
                  label: item.name,
                })) || []
              }
              placeholder="Select Roles"
              extra="*"
            />
          </FormControl>
          <FormControl w="50%">
            <MultiSelectWrapper
              id="ibcs"
              fieldName="ibcs"
              label="IBCs"
              options={ibcs?.map((item: IbcInterface) => {
                return { value: item.id, label: item.area_name };
              })}
              defaultValue={
                isEdit?.ibcs?.map((item: any) => ({
                  value: item.id,
                  label: item.area_name,
                })) || []
              }
              placeholder="Select IBCs"
              extra="*"
            />
          </FormControl>
        </Box>

        <Box display="flex" gap={15}>
          <FormControl w="50%">
            <MultiSelectWrapper
              id="projects"
              fieldName="projects"
              label="Projects"
              options={projects?.map((item: any) => {
                return { value: item?.id, label: item?.name };
              })}
              defaultValue={
                isEdit?.projects?.map((item: any) => ({
                  value: item.id,
                  label: item.name,
                })) || []
              }
              placeholder="Select Projects"
              extra="*"
            />
          </FormControl>
          <FormControl w="50%">
            <CheckboxWrapper
              id="is_active"
              fieldName="is_active"
              label="Is Active"
              extra="*"
              form
            />
          </FormControl>
        </Box>

        <Button variant="brand" onClick={handleSubmitClick} mr={3}>
          {isEdit ? "Save" : "Create"}
        </Button>
      </FormComponent>
    </>
  );
};

export default UserForm;
